#version 420

uniform sampler2D colorTexture;
uniform sampler2DShadow shadowMap;
// Im Moment werden maximal 4 Lichter unterstützt
// jede 1. Position enthält Lichtpositionen 3 floats)
// jede 2. Position enthält Lichtfarbe (3 floats)
// macht 3floats position + 3floats Farbe = 6floats, 6 * 4 Lichter = 24 floats
uniform float lights[24];
smooth in vec4 theColor;
in vec2 textureCoords;
in vec3 vertexWorldSpace;
in vec3 vertexNormal;
in vec4 SM_tex_coord;
// passed on from VS

varying vec3 vertexToCam;
out vec3 outputColor;
/**
* Berechnet die Intensität des reflektierten Lichts
* diese hängt auch von der Entfernung ab
*/
float calcIntensity(vec3 lightPosition){
    // die Attenuation-Werte geben an, wie sehr Entfernung die Lichtstärke vermindert
	// könnte man evtl. als uniforms einbinden. oder auch nicht :P
	float constantAttenuation = 0.3f;
    float linearAttenuation = 0.0000002f;
    float quadraticAttenuation = 0.00000001f;
    float dist = distance(vertexWorldSpace, lightPosition);
    float A = constantAttenuation + linearAttenuation * dist + quadraticAttenuation * pow(dist, 2);
    return A;
}

/*
float calcShadowIntensity(vec3 tex_coords)
{
	float sum = 0;
	float x, y;
	float numberOfEls = 0.0f;
	for (y = -1.0f; y <= 1.0f; y += 1.0f){
		for (x = -1.0f; x <= 1.0f; x += 1.0f){
			float depth = texture(shadowMap, tex_coords.xy + vec2(x, y)).r;
			// perform depth comparison
			float inShadow = (depth < tex_coords.z) ? 1.0f : 0.0f;
			sum += inShadow;
			numberOfEls++;
		}
	}

	float shadowCoeff = sum / numberOfEls;
	return shadowCoeff;
	//return textureProj(shadowMap, SM_tex_coord).r / 1.5;
	//return textureProj(shadowMap, vec4(loc.xy + offset * texmapscale * loc.w, loc.z, loc.w));
}
*/

void main()
{
    // ambient vielleicht per uniform setzbar machen?
	
    vec3 textureColor = texture(colorTexture, textureCoords).xyz;
    // perform perspective division
	vec3 tex_coords = SM_tex_coord.xyz/SM_tex_coord.w;

	// das ist die Variante ohne PCF:
	// float shadow = textureProj(shadowMap, SM_tex_coord, 4.0f).r;

	// Und hier mit PCF, hier 5x5 kernel
	float shadow = 0.0f;

	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(-2,-2), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(-2,-1), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(-2, 0), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(-2, 1), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(-2, 2), 1.0f).r;

	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(-1,-2), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(-1,-1), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(-1, 0), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(-1, 1), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(-1, 2), 1.0f).r;
	
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(0,-2), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(0,-1), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(0, 0), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(0, 1), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(0, 2), 1.0f).r;

	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(1,-2), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(1,-1), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(1, 0), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(1, 1), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(1, 2), 1.0f).r;

	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(2,-2), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(2,-1), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(2, 0), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(2, 1), 1.0f).r;
	shadow += textureProjOffset(shadowMap, SM_tex_coord, ivec2(2, 2), 1.0f).r;

	shadow /= 25; // 5x5 kernel = 25

	float corrShadowCoeff = shadow; // * 0.8f;

	vec3 ambient = vec3(0.3f);

    // für alle Lichter diffuse und specular berechnen
	vec3 summedLightColor;
    for(int i = 0; i < 24; i += 6){
        vec3 lightPosition = vec3(lights[i], lights[i+1], lights[i+2]);
        vec3 lightColor = vec3(lights[i+3], lights[i+4], lights[i+5]);
        float shininess = 15.0f;
        // specular color uniform machen
		vec3 specularColor = vec3(0.5f, 0.5f, 0.5f);
        // light intensity, um attenuation verringert:
		vec3 Il = lightColor / calcIntensity(lightPosition);
        vec3 L = normalize(lightPosition - vertexWorldSpace);
        vec3 V = normalize(vertexToCam);
        vec3 N = normalize(vertexNormal);
        float cosNL = max(dot(N, L), 0);
        vec3 H = (L + V) / length(L + V);
        float diffuseFactor = cosNL;
        float specularFactor = cosNL * pow( max(dot(N, H), 0), shininess);
        vec3 diffusePart = lightColor.rgb * diffuseFactor;
        vec3 specularPart = specularColor * specularFactor;
        summedLightColor += (diffusePart + specularPart) * Il  * corrShadowCoeff;
    }
    // darken fragment color if in shadow...
	//if(inShadow == 1){
	//
  //      summedLightColor = summedLightColor * shadowCoeff;
    // vec4(depth*depth*depth, depth*depth*depth, depth*depth*depth, 1.0f); // outputColor * depth; //calcShadowIntensity(tex_coords);
    //}

	outputColor = textureColor *( ambient + summedLightColor)  ;
}
