#version 330

// Im Moment werden maximal 4 Lichter unterstützt
// jede 1. Position enthält Lichtpositionen (3 floats)
// jede 2. Position enthält Lichtfarbe (3 floats)
// macht 1x position + 1x Farbe = 2, 2 * 4 Lichter = 8 x vec3
uniform vec3 lights[8];
uniform vec3 ambientColor;
smooth in vec4 theColor;
in vec3 vertexWorldSpace;
in vec3 vertexNormal;
varying vec3 vertexToCam;
out vec3 outputColor;
/**
* Berechnet die Intensität des reflektierten Lichts
* diese hängt auch von der Entfernung ab
*/
vec3 calcIntensity(vec3 lightPosition){
    // die Attenuation-Werte geben an, wie sehr Entfernung die Lichtstärke vermindert
	// könnte man evtl. als uniforms einbinden. oder auch nicht :P
	float constantAttenuation = 1.0f;
    float linearAttenuation = 0.0f;
    float quadraticAttenuation = 0.005f;
    float dist = distance(vertexWorldSpace, lightPosition);
    float A = constantAttenuation + linearAttenuation * dist + quadraticAttenuation * pow(dist, 2);
    vec3 color = lightColor.xyz;
    color.x = lightColor.x/A;
    color.y = lightColor.y/A;
    color.z = lightColor.z/A;
    return color;
}

void main()
{
	// für alle Lichter diffuse und specular berechnen
	vec3 summedLightColor;
	for(int i = 0; i < 4; i += 2){
		vec3 lightPosition = lights[i];
		vec3 lightColor = lights[i+1];

		 vec3 Il = calcIntensity(lightPosition);
		vec3 L = normalize(lightPosition - vertexWorldSpace);
		// Lambert
		float cosTheta = max(dot(vertexNormal, L), 0);
		vec3 diffLight = lightColor.rgb * cosTheta;
		vec3 lightIntensity = diffLight * Il;
		summedLightColor = lightIntensity;
	}
    // diese Werte werden für Lambert UND Blinn benötigt
	outputColor = summedLightColor;
}



